//
// This is a simple example project for the AMShellWrapper class.
//
// 2002-08-29  Andreas Mayer
// - first version
// 2004-06-15  Andreas Mayer
// - added stop function
// 2004-07-24  Andreas Mayer
// - added exception handling
// 2004-09-20  Andreas Mayer
// - write: uses replaceCharactersInRange:


#import "BannerController.h"

@interface BannerController (Private)
- (void)write:(NSString *)string;
- (void)processLaunchException:(NSException *)exception;
@end


@implementation BannerController

- (AMShellWrapper *)shellWrapper
{
	return shellWrapper;
}

- (void)setShellWrapper:(AMShellWrapper *)newShellWrapper
{
	id old = nil;
	
	if (newShellWrapper != shellWrapper) {
		old = shellWrapper;
		shellWrapper = [newShellWrapper retain];
		[old release];
	}
}


- (void)dealloc
{
	[shellWrapper release];
	[super dealloc];
}


- (IBAction)printBanner:(id)sender
{
	NSArray *arguments;
	
	[textOutlet setFont:[NSFont userFixedPitchFontOfSize:0.0]];
	
	arguments = [[textSource stringValue] componentsSeparatedByString:@" "];
	
	AMShellWrapper *wrapper = [[[AMShellWrapper alloc] initWithInputPipe:nil outputPipe:nil errorPipe:nil workingDirectory:@"." environment:nil arguments:arguments context:NULL] autorelease];
	[wrapper setDelegate:self];
	[self setShellWrapper:wrapper];
	
	NS_DURING
	if (shellWrapper) {
		[shellWrapper setOutputStringEncoding:NSASCIIStringEncoding];
		[shellWrapper startProcess];
	
		[self write:@"launched: "];
		[self write:[arguments objectAtIndex:0]];
		[self write:@"\r"];
	} else {
		[self write:@"Ops! Something went wrong.\r Was not able to execute command.\r"];
	}
NS_HANDLER
	NSLog(@"Caught %@: %@", [localException name], [localException reason]);
	[self processLaunchException:localException];
NS_ENDHANDLER
}

- (IBAction)stopTask:(id)sender
{
	[shellWrapper stopProcess];
}

- (void)write:(NSString *)string
{
	NSTextStorage *text = [textOutlet textStorage];
	[text replaceCharactersInRange:NSMakeRange([text length], 0) withString:string];
	[textOutlet scrollRangeToVisible:NSMakeRange([text length], 0)];
}


// ============================================================
// conforming to the AMShellWrapperDelegate protocol:
// ============================================================

// output from stdout
- (void)process:(AMShellWrapper *)wrapper appendOutput:(id)output
{
	[self write:output];
}

// output from stderr
- (void)process:(AMShellWrapper *)wrapper appendError:(NSString *)error
{
	[errorOutlet setString:[[errorOutlet string] stringByAppendingString:error]];
}

// This method is a callback which your controller can use to do other initialization
// when a process is launched.
- (void)processStarted:(AMShellWrapper *)wrapper
{
	[progressIndicator startAnimation:self];
	[runButton setTitle:@"Stop"];
	[runButton setAction:@selector(stopTask:)];
}

// This method is a callback which your controller can use to do other cleanup
// when a process is halted.
- (void)processFinished:(AMShellWrapper *)wrapper withTerminationStatus:(int)resultCode
{
	[self write:[NSString stringWithFormat:@"\rcommand finished. Result code: %i\r", resultCode]];
	[self setShellWrapper:nil];
	[textOutlet scrollRangeToVisible:NSMakeRange([[textOutlet string] length], 0)];
	[errorOutlet scrollRangeToVisible:NSMakeRange([[errorOutlet string] length], 0)];
	[runButton setEnabled:YES];
	[progressIndicator stopAnimation:self];
	[runButton setTitle:@"Execute"];
	[runButton setAction:@selector(printBanner:)];
}

- (void)processLaunchException:(NSException *)exception
{
	[self write:[NSString stringWithFormat:@"\rcaught %@ while executing command\r", [exception name]]];
	[textOutlet scrollRangeToVisible:NSMakeRange([[textOutlet string] length], 0)];
	[errorOutlet scrollRangeToVisible:NSMakeRange([[errorOutlet string] length], 0)];
	[runButton setEnabled:YES];
	[progressIndicator stopAnimation:self];
	[runButton setTitle:@"Execute"];
	[runButton setAction:@selector(printBanner:)];
	[self setShellWrapper:nil];
}


// -----------------------------------------------------------------------------
#pragma mark - NSApplicationDelegate
// -----------------------------------------------------------------------------

- (void)applicationDidFinishLaunching:(NSNotification *)notification
{
	NSFont *monospacedFont = [NSFont fontWithName:@"Monaco" size:12];
	NSDictionary *typingAttributes = [NSDictionary dictionaryWithObjectsAndKeys:monospacedFont, NSFontAttributeName, nil];
	NSTextStorage *text = [textOutlet textStorage];
	[text replaceCharactersInRange:NSMakeRange([text length], 0) withString:@" \n"];
	[text setAttributes:typingAttributes range:NSMakeRange(0, [text length])];
	[textOutlet setTypingAttributes:typingAttributes];
}

@end
